@@ -10,6 +10,7 @@ import redis
 import requests
 from decouple import config
 from melon_translate_client.utils.logging import log
+from redis.client import Pipeline
 from rest_framework.exceptions import ValidationError
 
 
@@ -51,8 +52,14 @@ class Client:
         self.cache_address = cache_address
         self.cache_port = cache_port
         self.cache = redis.Redis(
-            host=cache_address, port=cache_port, decode_responses=True
+            host=cache_address,
+            port=cache_port,
+            decode_responses=True,
+            socket_timeout=15,
+            socket_connect_timeout=15,
+            max_connections=7500,
         )
+        self.pipeline = self.cache.pipeline()
 
     @staticmethod
     def __stringify(collection: List[Any]) -> List[str]:
@@ -95,9 +102,17 @@ class Client:
         for _hash in hashes:
             self.cache.expire(_hash, Client.CACHE_GROUP_KEY_TTL)
 
-    def __set_view(self, language: str, view: str, ids: List[str]) -> Optional[bool]:
+    def _filter_url(self, language) -> str:
+        """Get service filter URL location.
+
+        :returns: url for the `filter` request
+        """
+        _url = urlparse(f"{self.address}:{self.port}/api/v1/translations/{language}/")
+        return _url.geturl()
+
+    def __set_view(self, language: str, view: str, ids: List[str]) -> Pipeline:
         """Set view with specified TTL."""
-        return self.cache.set(
+        return self.pipeline.set(
             f"{self.__hash(language, view)}",
             ",".join(self.__stringify(ids)),
             ex=Client.CACHE_VIEW_TTL,
@@ -105,9 +120,9 @@ class Client:
 
     def __set_occurrence(
         self, language: str, occurence: str, ids: List[str]
-    ) -> Optional[bool]:
+    ) -> Pipeline:
         """Set occurrence with specified TTL."""
-        return self.cache.set(
+        return self.pipeline.set(
             f"{self.__hash(language, occurence)}",
             ",".join(self.__stringify(ids)),
             ex=Client.CACHE_OCCURRENCE_TTL,
@@ -122,6 +137,52 @@ class Client:
 
         return {record.get("id"): record for record in _records}
 
+    def _get_all_pages(
+        self,
+        language: str,
+        views: Optional[List[str]] = None,
+        occurrences: Optional[List[str]] = None,
+        snake_keys: Optional[List[str]] = None,
+        page_size: int = 500,
+        page: int = 1,
+        chain_together: bool = True,
+    ) -> Optional[list]:
+        """
+        This method paginates response and returns list of lists of pages
+        """
+        url: str = self._filter_url(language)
+        params = self.__build_params(views, occurrences, snake_keys, page_size, page)
+
+        response = requests.get(url, params=params, timeout=Client.timeout).json()
+        if not response.get("results"):
+            return None
+
+        results = [
+            response,
+        ]
+        count = response.get("count")
+        next_page = response.get("links", {}).get("next")
+
+        while next_page:
+            response = requests.get(next_page, timeout=Client.timeout).json()
+            results.append(response)
+
+            page_num = response.get("pages").get("current")
+            next_page = response.get("links", {}).get("next")
+            log.info(f"Page {page_num} for {language} language.")
+
+        log.info(
+            f"There are no more pages. Total count is {count} translations with {params} parameters \n"
+            f"for {language} language."
+        )
+
+        if chain_together:
+            return list(
+                chain.from_iterable([result.get("results") for result in results])
+            )
+
+        return results
+
     def snake_key(self, language: str, key: str) -> Optional[dict]:
         """Retrieve a snake key."""
         _SNAKE_NAME_MAP = f"{language}_snake_name"
@@ -200,52 +261,6 @@ class Client:
 
         return record
 
-    def _get_all_pages(
-        self,
-        language: str,
-        views: Optional[List[str]] = None,
-        occurrences: Optional[List[str]] = None,
-        snake_keys: Optional[List[str]] = None,
-        page_size: int = 500,
-        page: int = 1,
-        chain_together: bool = True,
-    ) -> Optional[list]:
-        """
-        This method paginates response and returns list of lists of pages
-        """
-        url: str = self._filter_url(language)
-        params = self.__build_params(views, occurrences, snake_keys, page_size, page)
-
-        response = requests.get(url, params=params, timeout=Client.timeout).json()
-        if not response.get("results"):
-            return None
-
-        results = [
-            response,
-        ]
-        count = response.get("count")
-        next_page = response.get("links", {}).get("next")
-
-        while next_page:
-            response = requests.get(next_page, timeout=Client.timeout).json()
-            results.append(response)
-
-            page_num = response.get("pages").get("current")
-            next_page = response.get("links", {}).get("next")
-            log.info(f"Page {page_num} for {language} language.")
-
-        log.info(
-            f"There are no more pages. Total count is {count} translations with {params} parameters \n"
-            f"for {language} language."
-        )
-
-        if chain_together:
-            return list(
-                chain.from_iterable([result.get("results") for result in results])
-            )
-
-        return results
-
     def filter(
         self,
         language: str,
@@ -341,23 +356,23 @@ class Client:
                 _occurrences = _key.get("occurrences") or []
 
                 data = json.dumps(translate)
-                self.cache.set(_id, data)
+                self.pipeline.set(_id, data)
 
                 # NOTE: Build indexes for search/data access.
                 if _snake:
-                    self.cache.hset(_SNAKE_NAME_MAP, _snake, _id)
+                    self.pipeline.hset(_SNAKE_NAME_MAP, _snake, _id)
 
                 if _id_name:
-                    self.cache.hset(_ID_NAME_MAP, _id_name, _id)
+                    self.pipeline.hset(_ID_NAME_MAP, _id_name, _id)
 
                 for view in _views:
                     views[view].append(_id)
-                    self.cache.hset(_VIEWS_ID_MAP, _id, view)
+                    self.pipeline.hset(_VIEWS_ID_MAP, _id, view)
                     self.__expire(_VIEWS_ID_MAP)
 
                 for occurrence in _occurrences:
                     occurrences[occurrence].append(_id)
-                    self.cache.hset(_OCCURRENCES_ID_MAP, _id, occurrence)
+                    self.pipeline.hset(_OCCURRENCES_ID_MAP, _id, occurrence)
                     self.__expire(_OCCURRENCES_ID_MAP)
 
             # NOTE: Cache freshly retrieved items.
@@ -367,6 +382,7 @@ class Client:
             for occurrence, keys in occurrences.items():
                 self.__set_occurrence(language, occurrence, ids=keys)
 
+            self.pipeline.execute()
             return _cache
 
         result = _cache_warmup()
@@ -388,10 +404,24 @@ class Client:
         )
         return result
 
-    def _filter_url(self, language) -> str:
-        """Get service filter URL location.
+    def get_bootstrap_keys(self, language, view) -> dict:
+        """Get translations keys from redis for bootstrap init call"""
+        _SNAKE_NAME_MAP = f"{language}_snake_name"
+        keys_dict = {}
 
-        :returns: url for the `filter` request
-        """
-        _url = urlparse(f"{self.address}:{self.port}/api/v1/translations/{language}/")
-        return _url.geturl()
+        snake_names = self.cache.hgetall(_SNAKE_NAME_MAP)
+        temp_pipeline = self.cache.pipeline()
+
+        for value in snake_names.values():
+            temp_pipeline.get(value)
+
+        obj_list = temp_pipeline.execute()
+
+        for obj in obj_list:
+            obj = json.loads(obj)
+            keys = obj.get("key")
+            views = keys.get("views") or []
+            if view in views:
+                keys_dict[keys.get("snake_name")] = keys.get("id_name")
+
+        return keys_dict
